searchData={"items":[{"type":"module","title":"ErrorTracker","doc":"En Elixir-based built-in error tracking solution.\n\nThe main objectives behind this project are:\n\n* Provide a basic free error tracking solution: because tracking errors in\nyour application should be a requirement for almost any project, and helps to\nprovide quality and maintenance to your project.\n\n* Be easy to use: by providing plug-and-play integrations, documentation and a\nsimple UI to manage your errors.\n\n* Be as minimalistic as possible: you just need a database to store errors and\na Phoenix application if you want to inspect them via web. That's all.","ref":"ErrorTracker.html"},{"type":"module","title":"Requirements - ErrorTracker","doc":"ErrorTracker requires Elixir 1.15+, Ecto 3.11+, Phoenix LiveView 0.19+, and\nPostgreSQL, MySQL/MariaDB or SQLite3 as database.","ref":"ErrorTracker.html#module-requirements"},{"type":"module","title":"Integrations - ErrorTracker","doc":"We currently include integrations for what we consider the basic stack of\nan application: Phoenix, Plug, and Oban.\n\nHowever, we may continue working in adding support for more systems and\nlibraries in the future if there is enough interest from the community.\n\nIf you want to manually report an error, you can use the `ErrorTracker.report/3` function.","ref":"ErrorTracker.html#module-integrations"},{"type":"module","title":"Context - ErrorTracker","doc":"Aside from the information about each exception (kind, message, stack trace...)\nwe also store contexts.\n\nContexts are arbitrary maps that allow you to store extra information about an\nexception to be able to reproduce it later.\n\nEach integration includes a default context with useful information they\ncan gather, but aside from that, you can also add your own information. You can\ndo this in a per-process basis or in a per-call basis (or both).\n\nThere are some requirements on the type of data that can be included in the\ncontext, so we recommend taking a look at `set_context/1` documentation.\n\n**Per process**\n\nThis allows you to set a general context for the current process such as a Phoenix\nrequest or an Oban job. For example, you could include the following code in your\nauthentication Plug to automatically include the user ID in any error that is\ntracked during the Phoenix request handling.\n\n```elixir\nErrorTracker.set_context(%{user_id: conn.assigns.current_user.id})\n```\n\n**Per call**\n\nAs we had seen before, you can use `ErrorTracker.report/3` to manually report an\nerror. The third parameter of this function is optional and allows you to include\nextra context that will be tracked along with the error.","ref":"ErrorTracker.html#module-context"},{"type":"module","title":"Breadcrumbs - ErrorTracker","doc":"Aside from contextual information, it is sometimes useful to know in which points\nof your code the code was executed in a given request / process.\n\nUsing breadcrumbs allows you to add that information to any error generated and\nstored on a given process / request. And if you are using `Ash` or `Splode` their\nexceptions' breadcrumbs will be automatically populated.\n\nIf you want to add a breadcrumb in a point of your code you can do so:\n\n```elixir\nErrorTracker.add_breadcrumb(\"Executed my super secret code\")\n```\n\nBreadcrumbs can be viewed in the dashboard on the details page of an occurrence.","ref":"ErrorTracker.html#module-breadcrumbs"},{"type":"function","title":"ErrorTracker.add_breadcrumb/1","doc":"Adds a breadcrumb to the current process.\n\nThe new breadcrumb will be added as the most recent entry of the breadcrumbs\nlist.","ref":"ErrorTracker.html#add_breadcrumb/1"},{"type":"function","title":"Breadcrumbs limit - ErrorTracker.add_breadcrumb/1","doc":"Breadcrumbs are a powerful tool that allows to add an infinite number of\nentries. However, it is not recommended to store errors with an excessive\namount of breadcrumbs.\n\nAs they are stored as an array of strings under the hood, storing many\nentries per error can lead to some delays and using extra disk space on the\ndatabase.","ref":"ErrorTracker.html#add_breadcrumb/1-breadcrumbs-limit"},{"type":"type","title":"ErrorTracker.context/0","doc":"A map containing the relevant context for a particular error.","ref":"ErrorTracker.html#t:context/0"},{"type":"type","title":"ErrorTracker.exception/0","doc":"An `Exception` or a `{kind, payload}` tuple compatible with `Exception.normalize/3`.","ref":"ErrorTracker.html#t:exception/0"},{"type":"function","title":"ErrorTracker.get_breadcrumbs/0","doc":"Obtain the breadcrumbs of the current process.","ref":"ErrorTracker.html#get_breadcrumbs/0"},{"type":"function","title":"ErrorTracker.get_context/0","doc":"Obtain the context of the current process.","ref":"ErrorTracker.html#get_context/0"},{"type":"function","title":"ErrorTracker.report/3","doc":"Report an exception to be stored.\n\nReturns the occurrence stored or `:noop` if the ErrorTracker is disabled by\nconfiguration the exception has not been stored.\n\nAside from the exception, it is expected to receive the stack trace and,\noptionally, a context map which will be merged with the current process\ncontext.\n\nKeep in mind that errors that occur in Phoenix controllers, Phoenix LiveViews\nand Oban jobs are automatically reported. You will need this function only if you\nwant to report custom errors.\n\n```elixir\ntry do\n  # your code\ncatch\n  e ->\n    ErrorTracker.report(e, __STACKTRACE__)\nend\n```","ref":"ErrorTracker.html#report/3"},{"type":"function","title":"Exceptions - ErrorTracker.report/3","doc":"Exceptions can be passed in three different forms:\n\n* An exception struct: the module of the exception is stored along with\nthe exception message.\n\n* A `{kind, exception}` tuple in which case the information is converted to\nan Elixir exception (if possible) and stored.","ref":"ErrorTracker.html#report/3-exceptions"},{"type":"function","title":"ErrorTracker.resolve/1","doc":"Marks an error as resolved.\n\nIf an error is marked as resolved and it happens again, it will automatically\nappear as unresolved again.","ref":"ErrorTracker.html#resolve/1"},{"type":"function","title":"ErrorTracker.set_context/1","doc":"Sets the current process context.\n\nThe given context will be merged into the current process context. The given context\nmay override existing keys from the current process context.","ref":"ErrorTracker.html#set_context/1"},{"type":"function","title":"Context depth - ErrorTracker.set_context/1","doc":"You can store context on more than one level of depth, but take into account\nthat the merge operation is performed on the first level.\n\nThat means that any existing data on deep levels for he current context will\nbe replaced if the first level key is received on the new contents.","ref":"ErrorTracker.html#set_context/1-context-depth"},{"type":"function","title":"Content serialization - ErrorTracker.set_context/1","doc":"The content stored on the context should be serializable using the JSON library\nused by the application (usually `Jason`), so it is rather recommended to use\nprimitive types (strings, numbers, booleans...).\n\nIf you still need to pass more complex data types to your context, please test\nthat they can be encoded to JSON or storing the errors will fail. In the case\nof `Jason` that may require defining an Encoder for that data type if not\nincluded by default.","ref":"ErrorTracker.html#set_context/1-content-serialization"},{"type":"function","title":"ErrorTracker.unresolve/1","doc":"Marks an error as unresolved.","ref":"ErrorTracker.html#unresolve/1"},{"type":"behaviour","title":"ErrorTracker.Filter","doc":"Behaviour for sanitizing & modifying the error context before it's saved.\n\n    defmodule MyApp.ErrorFilter do\n      @behaviour ErrorTracker.Filter\n\n      @impl true\n      def sanitize(context) do\n        context # Modify the context object (add or remove fields as much as you need.)\n      end\n    end\n\nOnce implemented, include it in the ErrorTracker configuration:\n\n  config :error_tracker, filter: MyApp.Filter\n\nWith this configuration in place, the ErrorTracker will call `MyApp.Filter.sanitize/1` to get a context before\nsaving error occurrence.\n\n> #### A note on performance {: .warning}\n>\n> Keep in mind that the `sanitize/1` will be called in the context of the ErrorTracker itself.\n> Slow code will have a significant impact in the ErrorTracker performance. Buggy code can bring\n> the ErrorTracker process down.","ref":"ErrorTracker.Filter.html"},{"type":"callback","title":"ErrorTracker.Filter.sanitize/1","doc":"This function will be given an error context to inspect/modify before it's saved.","ref":"ErrorTracker.Filter.html#c:sanitize/1"},{"type":"behaviour","title":"ErrorTracker.Ignorer","doc":"Behaviour for ignoring errors.\n\nThe ErrorTracker tracks every error that happens in your application. In certain cases you may\nwant to ignore some errors and don't track them. To do so you can implement this behaviour.\n\n    defmodule MyApp.ErrorIgnorer do\n      @behaviour ErrorTracker.Ignorer\n\n      @impl true\n      def ignore?(error = %ErrorTracker.Error{}, context) do\n        # return true if the error should be ignored\n      end\n    end\n\nOnce implemented, include it in the ErrorTracker configuration:\n\n    config :error_tracker, ignorer: MyApp.ErrorIgnorer\n\nWith this configuration in place, the ErrorTracker will call `MyApp.ErrorIgnorer.ignore?/2` before\ntracking errors. If the function returns `true` the error will be ignored and won't be tracked.\n\n> #### A note on performance {: .warning}\n>\n> Keep in mind that the `ignore?/2` will be called in the context of the ErrorTracker itself.\n> Slow code will have a significant impact in the ErrorTracker performance. Buggy code can bring\n> the ErrorTracker process down.","ref":"ErrorTracker.Ignorer.html"},{"type":"callback","title":"ErrorTracker.Ignorer.ignore?/2","doc":"Decide wether the given error should be ignored or not.\n\nThis function receives both the current Error and context and should return a boolean indicating\nif it should be ignored or not. If the function returns true the error will be ignored, otherwise\nit will be tracked.","ref":"ErrorTracker.Ignorer.html#c:ignore?/2"},{"type":"behaviour","title":"ErrorTracker.Migration","doc":"Create and modify the database tables for ErrorTracker.","ref":"ErrorTracker.Migration.html"},{"type":"behaviour","title":"Usage - ErrorTracker.Migration","doc":"To use ErrorTracker migrations in your application you will need to generate\na regular `Ecto.Migration` that performs the relevant calls to `ErrorTracker.Migration`.\n\n```bash\nmix ecto.gen.migration add_error_tracker\n```\n\nOpen the generated migration file and call the `up` and `down` functions on\n`ErrorTracker.Migration`.\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddErrorTracker do\n  use Ecto.Migration\n\n  def up, do: ErrorTracker.Migration.up()\n  def down, do: ErrorTracker.Migration.down()\nend\n```\n\nThis will run every ErrorTracker migration for your database. You can now run the migration\nand perform the database changes:\n\n```bash\nmix ecto.migrate\n```\n\nAs new versions of ErrorTracker are released you may need to run additional migrations.\nTo do this you can follow the previous process and create a new migration:\n\n```bash\nmix ecto.gen.migration update_error_tracker_to_vN\n```\n\nOpen the generated migration file and call the `up` and `down` functions on the\n`ErrorTracker.Migration` passing the desired `version`.\n\n```elixir\ndefmodule MyApp.Repo.Migrations.UpdateErrorTrackerToVN do\n  use Ecto.Migration\n\n  def up, do: ErrorTracker.Migration.up(version: N)\n  def down, do: ErrorTracker.Migration.down(version: N)\nend\n```\n\nThen run the migrations to perform the database changes:\n\n```bash\nmix ecto.migrate\n```","ref":"ErrorTracker.Migration.html#module-usage"},{"type":"behaviour","title":"Custom prefix - PostgreSQL only - ErrorTracker.Migration","doc":"ErrorTracker supports namespacing its own tables using PostgreSQL schemas, also known\nas \"prefixes\" in Ecto. With prefixes your error tables can reside outside of your primary\nschema (which is usually named \"public\").\n\nTo use a prefix you need to specify it in your configuration:\n\n```elixir\nconfig :error_tracker, :prefix, \"custom_prefix\"\n```\n\nMigrations will automatically create the database schema for you. If the schema does already exist\nthe migration may fail when trying to recreate it. In such cases you can instruct ErrorTracker\nnot to create the schema again:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddErrorTracker do\n  use Ecto.Migration\n\n  def up, do: ErrorTracker.Migration.up(create_schema: false)\n  def down, do: ErrorTracker.Migration.down()\nend\n```\n\nYou can also override the configured prefix in the migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddErrorTracker do\n  use Ecto.Migration\n\n  def up, do: ErrorTracker.Migration.up(prefix: \"custom_prefix\")\n  def down, do: ErrorTracker.Migration.down(prefix: \"custom_prefix\")\nend\n```","ref":"ErrorTracker.Migration.html#module-custom-prefix-postgresql-only"},{"type":"callback","title":"ErrorTracker.Migration.current_version/1","doc":"","ref":"ErrorTracker.Migration.html#c:current_version/1"},{"type":"function","title":"ErrorTracker.Migration.down/1","doc":"","ref":"ErrorTracker.Migration.html#down/1"},{"type":"callback","title":"ErrorTracker.Migration.down/1","doc":"","ref":"ErrorTracker.Migration.html#c:down/1"},{"type":"function","title":"ErrorTracker.Migration.migrated_version/1","doc":"","ref":"ErrorTracker.Migration.html#migrated_version/1"},{"type":"function","title":"ErrorTracker.Migration.up/1","doc":"","ref":"ErrorTracker.Migration.html#up/1"},{"type":"callback","title":"ErrorTracker.Migration.up/1","doc":"","ref":"ErrorTracker.Migration.html#c:up/1"},{"type":"module","title":"ErrorTracker.Telemetry","doc":"Telemetry events of ErrorTracker.\n\nErrorTracker emits some events to allow third parties to receive information\nof errors and occurrences stored.","ref":"ErrorTracker.Telemetry.html"},{"type":"module","title":"Error events - ErrorTracker.Telemetry","doc":"Those occur during the life cycle of an error:\n\n* `[:error_tracker, :error, :new]`: is emitted when a new error is stored and\nno previous occurrences were known.\n\n* `[:error_tracker, :error, :resolved]`: is emitted when a new error is marked\nas resolved on the UI.\n\n* `[:error_tracker, :error, :unresolved]`: is emitted when a new error is\nmarked as unresolved on the UI or a new occurrence is registered, moving the\nerror to the unresolved state.","ref":"ErrorTracker.Telemetry.html#module-error-events"},{"type":"module","title":"Occurrence events - ErrorTracker.Telemetry","doc":"There is only one event emitted for occurrences:\n\n* `[:error_tracker, :occurrence, :new]`: is emitted when a new occurrence is\nstored.","ref":"ErrorTracker.Telemetry.html#module-occurrence-events"},{"type":"module","title":"Measures and metadata - ErrorTracker.Telemetry","doc":"Each event is emitted with some measures and metadata, which can be used to\nreceive information without having to query the database again:\n\n| event                                   | measures       | metadata      |\n| --------------------------------------- | -------------- | ------------- |\n| `[:error_tracker, :error, :new]`        | `:system_time` | `:error`      |\n| `[:error_tracker, :error, :unresolved]` | `:system_time` | `:error`      |\n| `[:error_tracker, :error, :resolved]`   | `:system_time` | `:error`      |\n| `[:error_tracker, :occurrence, :new]`   | `:system_time` | `:occurrence` |","ref":"ErrorTracker.Telemetry.html#module-measures-and-metadata"},{"type":"module","title":"ErrorTracker.Integrations.Oban","doc":"Integration with Oban.","ref":"ErrorTracker.Integrations.Oban.html"},{"type":"module","title":"How to use it - ErrorTracker.Integrations.Oban","doc":"It is a plug and play integration: as long as you have Oban installed the\nErrorTracker will receive and store the errors as they are reported.","ref":"ErrorTracker.Integrations.Oban.html#module-how-to-use-it"},{"type":"module","title":"How it works - ErrorTracker.Integrations.Oban","doc":"It works using Oban's Telemetry events, so you don't need to modify anything\non your application.\n\n> #### A note on errors grouping {: .warning}\n>\n> All errors reported using `:error` or `{:error, any()}` as the output of\n> your `perform/2` worker function are going to be grouped together (one group\n> of those of errors per worker).\n>\n> The reason of that behaviour is that those errors do not generate an exception,\n> so no stack trace is detected and they are stored as happening in the same\n> place.\n>\n> If you want errors of your workers to be grouped as you may expect on other\n> integrations, you should raise exceptions to report errors instead of gracefully\n> returning an error value.","ref":"ErrorTracker.Integrations.Oban.html#module-how-it-works"},{"type":"module","title":"Default context - ErrorTracker.Integrations.Oban","doc":"By default we store some context for you on errors generated in an Oban\nprocess:\n\n* `job.id`: the unique ID of the job.\n\n* `job.worker`: the name of the worker module.\n\n* `job.queue`: the name of the queue in which the job was inserted.\n\n* `job.args`: the arguments of the job being executed.\n\n* `job.priority`: the priority of the job.\n\n* `job.attempt`: the number of attempts performed for the job.","ref":"ErrorTracker.Integrations.Oban.html#module-default-context"},{"type":"module","title":"ErrorTracker.Integrations.Phoenix","doc":"Integration with Phoenix applications.","ref":"ErrorTracker.Integrations.Phoenix.html"},{"type":"module","title":"How to use it - ErrorTracker.Integrations.Phoenix","doc":"It is a plug and play integration: as long as you have Phoenix installed the\nErrorTracker will receive and store the errors as they are reported.\n\nIt also collects the exceptions that raise on your LiveView modules.","ref":"ErrorTracker.Integrations.Phoenix.html#module-how-to-use-it"},{"type":"module","title":"How it works - ErrorTracker.Integrations.Phoenix","doc":"It works using Phoenix's Telemetry events, so you don't need to modify\nanything on your application.","ref":"ErrorTracker.Integrations.Phoenix.html#module-how-it-works"},{"type":"module","title":"Errors on the Endpoint - ErrorTracker.Integrations.Phoenix","doc":"This integration only catches errors that raise after the requests hits your\nRouter. That means that an exception on a plug defined on your Endpoint will\nnot be reported.\n\nIf you want to also catch those errors, we recommend you to set up the\n`ErrorTracker.Integrations.Plug` integration too.","ref":"ErrorTracker.Integrations.Phoenix.html#module-errors-on-the-endpoint"},{"type":"module","title":"Default context - ErrorTracker.Integrations.Phoenix","doc":"For errors that are reported when executing regular HTTP requests (the ones\nthat go to Controllers), the context added by default is the same that you\ncan find on the `ErrorTracker.Integrations.Plug` integration.\n\nAs for exceptions generated in LiveView processes, we collect some special\ninformation on the context:\n\n* `live_view.view`: the LiveView module itself,\n\n* `live_view.uri`: last URI that loaded the LiveView (available when the\n`handle_params` function is invoked).\n\n* `live_view.params`: the params received by the LiveView (available when the\n`handle_params` function is invoked).\n\n* `live_view.event`: last event received by the LiveView (available when the\n`handle_event` function is invoked).\n\n* `live_view.event_params`: last event params received by the LiveView\n(available when the `handle_event` function is invoked).","ref":"ErrorTracker.Integrations.Phoenix.html#module-default-context"},{"type":"module","title":"ErrorTracker.Integrations.Plug","doc":"Integration with Plug applications.","ref":"ErrorTracker.Integrations.Plug.html"},{"type":"module","title":"How to use it - ErrorTracker.Integrations.Plug","doc":"","ref":"ErrorTracker.Integrations.Plug.html#module-how-to-use-it"},{"type":"module","title":"Plug applications - ErrorTracker.Integrations.Plug","doc":"The way to use this integration is by adding it to either your `Plug.Builder`\nor `Plug.Router`:\n\n```elixir\ndefmodule MyApp.Router do\n  use Plug.Router\n  use ErrorTracker.Integrations.Plug\n\n  ...\nend\n```","ref":"ErrorTracker.Integrations.Plug.html#module-plug-applications"},{"type":"module","title":"Phoenix applications - ErrorTracker.Integrations.Plug","doc":"There is a particular use case which can be useful when running a Phoenix\nweb application.\n\nIf you want to record exceptions that may occur in your application's endpoint\nbefore reaching your router (for example, in any plug like the ones decoding\ncookies of body contents) you may want to add this integration too:\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint\n  use ErrorTracker.Integrations.Plug\n\n  ...\nend\n```","ref":"ErrorTracker.Integrations.Plug.html#module-phoenix-applications"},{"type":"module","title":"Default context - ErrorTracker.Integrations.Plug","doc":"By default we store some context for you on errors generated during a Plug\nrequest:\n\n* `request.host`: the `conn.host` value.\n\n* `request.ip`: the IP address that initiated the request. It includes parsing\nproxy headers\n\n* `request.method`: the HTTP method of the request.\n\n* `request.path`: the path of the request.\n\n* `request.query`: the query string of the request.\n\n* `request.params`: parsed params of the request (only available if they have\nbeen fetched and parsed as part of the Plug pipeline).\n\n* `request.headers`: headers received on the request. All headers are included\nby default except for the `Cookie` ones, as they may include large and\nsensitive content like sessions.","ref":"ErrorTracker.Integrations.Plug.html#module-default-context"},{"type":"module","title":"ErrorTracker.Plugins.Pruner","doc":"Periodically delete resolved errors based on their age.\n\nPruning allows you to keep your database size under control by removing old errors that are not\nneeded anymore.","ref":"ErrorTracker.Plugins.Pruner.html"},{"type":"module","title":"Using the pruner - ErrorTracker.Plugins.Pruner","doc":"To enable the pruner you must register the plugin in the ErrorTracker configuration. This will use\nthe default options, which is to prune errors resolved after 24 hours.\n\n    config :error_tracker,\n      plugins: [ErrorTracker.Plugins.Pruner]\n\nYou can override the default options by passing them as an argument when registering the plugin.\n\n    config :error_tracker,\n      plugins: [{ErrorTracker.Plugins.Pruner, max_age: :timer.minutes(30)}]","ref":"ErrorTracker.Plugins.Pruner.html#module-using-the-pruner"},{"type":"module","title":"Options - ErrorTracker.Plugins.Pruner","doc":"- `:limit`  - the maximum number of errors to prune on each execution. Occurrences are removed\n  along the errors. The default is 200 to prevent timeouts and unnecesary database load.\n\n- `:max_age` - the number of milliseconds after a resolved error may be pruned. The default is 24\n  hours.\n\n- `:interval` - the interval in milliseconds between pruning runs. The default is 30 minutes.\n\nYou may find the `:timer` module functions useful to pass readable values to the `:max_age` and\n`:interval` options.","ref":"ErrorTracker.Plugins.Pruner.html#module-options"},{"type":"module","title":"Manual pruning - ErrorTracker.Plugins.Pruner","doc":"In certain cases you may prefer to run the pruner manually. This can be done by calling the\n`prune_errors/2` function from your application code. This function supports the `:limit` and\n`:max_age` options as described above.\n\nFor example, you may call this function from an Oban worker so you can leverage Oban's cron\ncapabilities and have a more granular control over when pruning is run.\n\n    defmodule MyApp.ErrorPruner do\n      use Oban.Job\n\n      def perform(%Job{}) do\n        ErrorTracker.Plugins.Pruner.prune_errors(limit: 10_000, max_age: :timer.minutes(60))\n      end\n    end","ref":"ErrorTracker.Plugins.Pruner.html#module-manual-pruning"},{"type":"function","title":"ErrorTracker.Plugins.Pruner.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ErrorTracker.Plugins.Pruner.html#child_spec/1"},{"type":"function","title":"ErrorTracker.Plugins.Pruner.prune_errors/1","doc":"Prunes resolved errors.\n\nYou do not need to use this function if you activate the Pruner plugin. This function is exposed\nonly for advanced use cases and Oban integration.","ref":"ErrorTracker.Plugins.Pruner.html#prune_errors/1"},{"type":"function","title":"Options - ErrorTracker.Plugins.Pruner.prune_errors/1","doc":"- `:limit`  - the maximum number of errors to prune on each execution. Occurrences are removed\n  along the errors. The default is 200 to prevent timeouts and unnecesary database load.\n\n- `:max_age` - the number of milliseconds after a resolved error may be pruned. The default is 24\n  hours. You may find the `:timer` module functions useful to pass readable values to this option.","ref":"ErrorTracker.Plugins.Pruner.html#prune_errors/1-options"},{"type":"function","title":"ErrorTracker.Plugins.Pruner.start_link/1","doc":"","ref":"ErrorTracker.Plugins.Pruner.html#start_link/1"},{"type":"module","title":"ErrorTracker.Error","doc":"Schema to store an error or exception recorded by ErrorTracker.\n\nIt stores a kind, reason and source code location to generate a unique\nfingerprint that can be used to avoid duplicates.\n\nThe fingerprint currently does not include the reason itself because it can\ncontain specific details that can change on the same error depending on\nruntime conditions.","ref":"ErrorTracker.Error.html"},{"type":"function","title":"ErrorTracker.Error.has_source_info?/1","doc":"Returns if the Error has information of the source or not.\n\nErrors usually have information about in which line and function occurred, but\nin some cases (like an Oban job ending with `{:error, any()}`) we cannot get\nthat information and no source is stored.","ref":"ErrorTracker.Error.html#has_source_info?/1"},{"type":"type","title":"ErrorTracker.Error.t/0","doc":"","ref":"ErrorTracker.Error.html#t:t/0"},{"type":"module","title":"ErrorTracker.Occurrence","doc":"Schema to store a particular instance of an error in a given time.\n\nIt contains all the metadata available about the moment and the environment\nin which the exception raised.","ref":"ErrorTracker.Occurrence.html"},{"type":"type","title":"ErrorTracker.Occurrence.t/0","doc":"","ref":"ErrorTracker.Occurrence.html#t:t/0"},{"type":"module","title":"ErrorTracker.Stacktrace","doc":"An Stacktrace contains the information about the execution stack for a given\noccurrence of an exception.","ref":"ErrorTracker.Stacktrace.html"},{"type":"function","title":"ErrorTracker.Stacktrace.new/1","doc":"","ref":"ErrorTracker.Stacktrace.html#new/1"},{"type":"function","title":"ErrorTracker.Stacktrace.source/1","doc":"Source of the error stack trace.\n\nThe first line matching the client application. If no line belongs to the current\napplication, just the first line.","ref":"ErrorTracker.Stacktrace.html#source/1"},{"type":"type","title":"ErrorTracker.Stacktrace.t/0","doc":"","ref":"ErrorTracker.Stacktrace.html#t:t/0"},{"type":"module","title":"ErrorTracker.Stacktrace.Line","doc":"","ref":"ErrorTracker.Stacktrace.Line.html"},{"type":"module","title":"ErrorTracker.Web","doc":"ErrorTracker includes a dashboard to view and inspect errors that occurred\non your application and are already stored in the database.\n\nIn order to use it, you need to add the following to your Phoenix's\n`router.ex` file:\n\n```elixir\ndefmodule YourAppWeb.Router do\n  use Phoenix.Router\n  use ErrorTracker.Web, :router\n\n  ...\n\n  scope \"/\" do\n    ...\n\n    error_tracker_dashboard \"/errors\"\n  end\nend\n```\n\nThis will add the routes needed for ErrorTracker's dashboard to work.\n\n**Note:** when adding the dashboard routes, make sure you do it in an scope that\nhas CSRF protection (usually the `:browser` pipeline in most projects), as\notherwise you may experience LiveView issues like crashes and redirections.","ref":"ErrorTracker.Web.html"},{"type":"module","title":"Security considerations - ErrorTracker.Web","doc":"Errors may contain sensitive information, like IP addresses, users information\nor even passwords sent on forms!\n\nSecuring your dashboard is an important part of integrating ErrorTracker on\nyour project.\n\nIn order to do so, we recommend implementing your own security mechanisms in\nthe form of a mount hook and pass it to the `error_tracker_dashboard` macro\nusing the `on_mount` option.\n\nYou can find more details on\n`ErrorTracker.Web.Router.error_tracker_dashboard/2`.","ref":"ErrorTracker.Web.html#module-security-considerations"},{"type":"module","title":"Static assets - ErrorTracker.Web","doc":"Static assets (CSS and JS) are inlined during the compilation. If you have\na [Content Security Policy](https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP)\nbe sure to allow inline styles and scripts.\n\nTo do this, ensure that your `style-src` and `script-src` policies include the\n`unsafe-inline` value.","ref":"ErrorTracker.Web.html#module-static-assets"},{"type":"module","title":"LiveView socket options - ErrorTracker.Web","doc":"By default the library expects you to have your LiveView socket at `/live` and\nusing `websocket` transport.\n\nIf that's not the case, you can configure it adding the following\nconfiguration to your app's config files:\n\n```elixir\nconfig :error_tracker,\n  socket: [\n    path: \"/my-custom-socket-path\"\n    transport: :longpoll # (accepted values are :longpoll or :websocket)\n  ]\n```","ref":"ErrorTracker.Web.html#module-liveview-socket-options"},{"type":"module","title":"ErrorTracker.Web.Router","doc":"ErrorTracker UI integration into your application's router.","ref":"ErrorTracker.Web.Router.html"},{"type":"macro","title":"ErrorTracker.Web.Router.error_tracker_dashboard/2","doc":"Creates the routes needed to use the `ErrorTracker` web interface.\n\nIt requires a path in which you are going to serve the web interface.\n\nIn order to work properly, the route should be in a scope with CSRF protection\n(usually the `:browser` pipeline).","ref":"ErrorTracker.Web.Router.html#error_tracker_dashboard/2"},{"type":"macro","title":"Security considerations - ErrorTracker.Web.Router.error_tracker_dashboard/2","doc":"The dashboard inlines both the JS and CSS assets. This means that, if your\napplication has a Content Security Policy, you need to specify the\n`csp_nonce_assign_key` option, which is explained below.","ref":"ErrorTracker.Web.Router.html#error_tracker_dashboard/2-security-considerations"},{"type":"macro","title":"Options - ErrorTracker.Web.Router.error_tracker_dashboard/2","doc":"* `on_mount`: a list of mount hooks to use before invoking the dashboard\nLiveView views.\n\n* `as`: a session name to use for the dashboard LiveView session. By default\nit uses `:error_tracker_dashboard`.\n\n* `csp_nonce_assign_key`: an assign key to find the CSP nonce value used for assets.\nSupports either `atom()` or a map of type\n`%{optional(:img) => atom(), optional(:script) => atom(), optional(:style) => atom()}`","ref":"ErrorTracker.Web.Router.html#error_tracker_dashboard/2-options"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThis guide is an introduction to ErrorTracker, an Elixir-based built-in error tracking solution. ErrorTracker provides a basic and free error-tracking solution integrated in your own application. It is designed to be easy to install and easy to use so you can integrate it in your existing project with minimal changes. The only requirement is a relational database in which errors will be tracked.\n\nIn this guide we will learn how to install ErrorTracker in an Elixir project so you can start reporting errors as soon as possible. We will also cover more advanced topics such as how to report custom errors and how to add extra context to reported errors.\n\n**This guide requires you to have set up Ecto with PostgreSQL, MySQL/MariaDB or SQLite3 beforehand.**","ref":"getting-started.html"},{"type":"extras","title":"Installing ErrorTracker as a dependency - Getting Started","doc":"The first step to add ErrorTracker to your application is to declare the package as a dependency in your `mix.exs` file:\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:error_tracker, \"~> 0.5\"}\n  ]\nend\n```\n\nOnce ErrorTracker is declared as a dependency of your application, you can install it with the following command:\n\n```bash\nmix deps.get\n```","ref":"getting-started.html#installing-errortracker-as-a-dependency"},{"type":"extras","title":"Configuring ErrorTracker - Getting Started","doc":"ErrorTracker needs a few configuration options to work. This configuration should be added to your `config/config.exs` file:\n\n```elixir\nconfig :error_tracker,\n  repo: MyApp.Repo,\n  otp_app: :my_app,\n  enabled: true\n```\n\nThe `:repo` option specifies the repository that will be used by ErrorTracker. You can use your regular application repository or a different one if you prefer to keep errors in a different database.\n\nThe `:otp_app` option specifies your application name. When an error occurs, ErrorTracker will use this information to understand which parts of the stack trace belong to your application and which parts belong to third-party dependencies. This allows you to filter in-app vs third-party frames when viewing errors.\n\nThe `:enabled` option (defaults to `true` if not present) allows to disable the ErrorTracker on certain environments. This is useful to avoid filling your dev database with errors, for example.","ref":"getting-started.html#configuring-errortracker"},{"type":"extras","title":"Setting up the database - Getting Started","doc":"Since ErrorTracker stores errors in the database you must create a database migration to add the required tables:\n\n```\nmix ecto.gen.migration add_error_tracker\n```\n\nOpen the generated migration and call the `up` and `down` functions on `ErrorTracker.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddErrorTracker do\n  use Ecto.Migration\n\n  def up, do: ErrorTracker.Migration.up(version: 4)\n\n  # We specify `version: 1` in `down`, to ensure we remove all migrations.\n  def down, do: ErrorTracker.Migration.down(version: 1)\nend\n```\n\nYou can run the migration and apply the database changes with the following command:\n\n```bash\nmix ecto.migrate\n```\n\nFor more information about how to handle migrations, take a look at the `ErrorTracker.Migration` module docs.","ref":"getting-started.html#setting-up-the-database"},{"type":"extras","title":"Automatic error tracking - Getting Started","doc":"At this point, ErrorTracker is ready to track errors. It will automatically start when your application boots and track errors that occur in your Phoenix controllers, Phoenix LiveViews and Oban jobs. The `ErrorTracker.Integrations.Phoenix` and `ErrorTracker.Integrations.Oban` provide detailed information about how this works.\n\nIf your application uses Plug but not Phoenix, you will need to add the relevant integration in your `Plug.Builder` or `Plug.Router` module.\n\n```elixir\ndefmodule MyApp.Router do\n  use Plug.Router\n  use ErrorTracker.Integrations.Plug\n\n  # Your code here\nend\n```\n\nThis is also required if you want to track errors that happen in your Phoenix endpoint, before the Phoenix router starts handling the request. Keep in mind that this won't be needed in most cases as endpoint errors are infrequent.\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint\n  use ErrorTracker.Integrations.Plug\n\n  # Your code here\nend\n```\n\nYou can learn more about this in the `ErrorTracker.Integrations.Plug` module documentation.","ref":"getting-started.html#automatic-error-tracking"},{"type":"extras","title":"Error context - Getting Started","doc":"The default integrations include some additional context when tracking errors. You can take a look at the relevant integration modules to see what is being tracked out of the box.\n\nIn certain cases, you may want to include some additional information when tracking errors. For example it may be useful to track the user ID that was using the application when an error happened. Fortunately, ErrorTracker allows you to enrich the default context with custom information.\n\nThe `ErrorTracker.set_context/1` function stores the given context in the current process so any errors that occur in that process (for example, a Phoenix request or an Oban job) will include this given context along with the default integration context.\n\nThere are some requirements on the type of data that can be included in the context, so we recommend taking a look at `ErrorTracker.set_context/1` documentation\n\n```elixir\nErrorTracker.set_context(%{user_id: conn.assigns.current_user.id})\n```\n\nYou may also want to sanitize or filter out some information from the context before saving it. To do that you can take a look at the `ErrorTracker.Filter` behaviour.","ref":"getting-started.html#error-context"},{"type":"extras","title":"Manual error tracking - Getting Started","doc":"If you want to report custom errors that fall outside the default integration scope, you may use `ErrorTracker.report/2`. This allows you to report an exception yourself:\n\n```elixir\ntry do\n  # your code\ncatch\n  e ->\n    ErrorTracker.report(e, __STACKTRACE__)\nend\n```\n\nYou can also use `ErrorTracker.report/3` and set some custom context that will be included along with the reported error.","ref":"getting-started.html#manual-error-tracking"},{"type":"extras","title":"Web UI - Getting Started","doc":"ErrorTracker also provides a dashboard built with Phoenix LiveView that can be used to see and manage the recorded errors.\n\nThis is completely optional, and you can find more information about it in the `ErrorTracker.Web` module documentation.","ref":"getting-started.html#web-ui"},{"type":"extras","title":"Notifications - Getting Started","doc":"Currently ErrorTracker does not support notifications out of the box.\n\nHowever, it provides some detailed Telemetry events that you may use to implement your own notifications following your custom rules and notification channels.\n\nIf you want to take a look at the events you can attach to, take a look at `ErrorTracker.Telemetry` module documentation.","ref":"getting-started.html#notifications"},{"type":"extras","title":"Pruning resolved errors - Getting Started","doc":"By default errors are kept in the database indefinitely. This is not ideal for production\nenvironments where you may want to prune old errors that have been resolved.\n\nThe `ErrorTracker.Plugins.Pruner` module provides automatic pruning functionality with a configurable\ninterval and error age.","ref":"getting-started.html#pruning-resolved-errors"},{"type":"extras","title":"Ignoring errors - Getting Started","doc":"ErrorTracker tracks every error by default. In certain cases some errors may be expected or just not interesting to track.\nErrorTracker provides functionality that allows you to ignore errors based on their attributes and context.\n\nTake a look at the `ErrorTracker.Ignorer` behaviour for more information about how to implement your own ignorer.","ref":"getting-started.html#ignoring-errors"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.0"}}